@@ -134,7 +134,6 @@ class WSTunnelTestCase(AsyncTestCase, LogTrapTestCase):
         super(WSTunnelTestCase, self).tearDown()
         clean_logging([self.log_file, self.pid_file])
 
-
     def on_response_received(self, response):
         """
         Callback invoked when response is received
@@ -142,6 +141,13 @@ class WSTunnelTestCase(AsyncTestCase, LogTrapTestCase):
         self.assertEqual(self.message.upper(), response)
         self.stop()
 
+    def on_response_resend(self, response):
+        """
+        Callback invoked when response is received. It resends the message, so that there will be an infinite loop.
+        """
+        self.assertEqual(self.message.upper(), response)
+        self.client.write(self.message)
+
     def test_request_response(self):
         """
         Test a simple request/response chat through the websocket tunnel
@@ -239,6 +245,42 @@ class WSTunnelTestCase(AsyncTestCase, LogTrapTestCase):
             self.srv_tun.remove_proxy(key)
             self.assertEqual(0, len(self.srv_tun.proxies))
 
+    def test_server_peer_connection_drop_issue_6(self):
+        """
+        Tests dropping peer connection server side. This test addresses issue #6
+        """
+
+        def on_close(*args):
+            raise Exception("CLOSED")
+
+        self.client.handle_close = on_close
+        self.client.send_message(self.message, handle_response=self.on_response_resend)
+        self.srv.stop()
+        try:
+            self.wait(timeout=ASYNC_TIMEOUT)
+        except Exception as e:
+            self.assertEqual("CLOSED", str(e))
+
+    def test_server_tunnel_connection_drop(self):
+        """
+        Tests dropping connection server side by shutting down the WebSocket tunnel server
+        """
+        def on_close(*args):
+            raise Exception("CLOSED")
+
+        self.client.handle_close = on_close
+        self.client.send_message(self.message, handle_response=self.on_response_resend)
+        self.srv_tun.stop()
+        try:
+            self.wait(timeout=ASYNC_TIMEOUT)
+        except Exception as e:
+            # Possibile cases (timing matters here):
+            # 1. Connection is lost when the client already sent next message --> CLOSED since the
+            # endpoint notified the client just in time.
+            # 2. Connection is lost when the client did not send the next message --> ASYNC_TIMEOUT since no endpoint
+            # responding
+            self.assertTrue(str(e).lower() in ("closed", "async operation timed out after %d seconds" % ASYNC_TIMEOUT))
+
 
 class WSTunnelSSLTestCase(WSTunnelTestCase):
     """
